package jp.sf.amateras.xmleditor.editors;

import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.Reader;
import java.net.URL;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import jp.sf.amateras.xmleditor.XMLEditorPlugin;

import org.eclipse.jface.text.ITextViewer;
import org.eclipse.jface.text.contentassist.CompletionProposal;
import org.eclipse.jface.text.contentassist.ICompletionProposal;
import org.eclipse.jface.text.contentassist.IContentAssistProcessor;
import org.eclipse.jface.text.contentassist.IContextInformation;
import org.eclipse.jface.text.contentassist.IContextInformationValidator;
import org.eclipse.swt.graphics.Image;

import com.wutka.dtd.DTD;
import com.wutka.dtd.DTDAttribute;
import com.wutka.dtd.DTDDecl;
import com.wutka.dtd.DTDElement;
import com.wutka.dtd.DTDEmpty;
import com.wutka.dtd.DTDItem;
import com.wutka.dtd.DTDParseException;
import com.wutka.dtd.DTDParser;

public class XMLAssistProcessor implements IContentAssistProcessor {
	
	private Image elementImage = XMLEditorPlugin.getImageDescriptor("icons/element.gif").createImage();
	private Image attributeImage = XMLEditorPlugin.getImageDescriptor("icons/attribute.gif").createImage();
	
	private String dtd = "";
	private List tagList = new ArrayList();
	
	public ICompletionProposal[] computeCompletionProposals(ITextViewer viewer, int offset) {
		List proposals = new ArrayList();
		String source = viewer.getDocument().get();
		
		// DTD擾iOƈĂꍇ̂DTDēǂݍ݁j
		String dtd = getDTD(source);
		if(!dtd.equals(this.dtd)){
			this.dtd = dtd;
			try {
				URL url = new URL(dtd);
				InputStream in = url.openStream();
				Reader reader = new InputStreamReader(in);
				refresh(reader);
			} catch(Exception ex){
				ex.printStackTrace();
			}
		}
		
		AssistInfo info = getAssistInfo(source, offset);
		
		if(info.matchString.startsWith("<")){
			// ^O̕⊮
			for(int i=0;i<tagList.size();i++){
				TagInfo tagInfo = (TagInfo)tagList.get(i);
				if(("<" + tagInfo.name).startsWith(info.matchString)){
					String replaceString = "<" + tagInfo.name;
					String displayString = tagInfo.name;
					int position = 0;
					// K{⊮
					for(int j=0;j<tagInfo.attributes.size();j++){
						AttributeInfo attrInfo = (AttributeInfo)tagInfo.attributes.get(j);
						if(attrInfo.required){
							replaceString = replaceString + " " + attrInfo.name + "=\"\"";
						}
					}
					if(tagInfo.hasBody){
						replaceString = replaceString + "></" + tagInfo.name + ">";
						position = replaceString.length() - tagInfo.name.length() - 3;
					} else {
						replaceString = replaceString + "/>";
						position = replaceString.length();
					}
					proposals.add(new CompletionProposal(replaceString, 
							offset - info.matchString.length(), info.matchString.length(), 
							position, elementImage, displayString, null, null));
				}
			}
		} else if(info.tagName.length() > 0){
			// ̕⊮
			for(int i=0;i<tagList.size();i++){
				TagInfo tagInfo = (TagInfo)tagList.get(i);
				if(tagInfo.name.equals(info.tagName)){
					for(int j=0;j<tagInfo.attributes.size();j++){
						AttributeInfo attrInfo = (AttributeInfo)tagInfo.attributes.get(j);
						if((attrInfo.name).startsWith(info.matchString)){
							String replaceString = attrInfo.name + "=\"\"";
							String displayString = attrInfo.name;
							int position = replaceString.length() - 1;
							proposals.add(new CompletionProposal(replaceString, 
									offset - info.matchString.length(), info.matchString.length(), 
									position, attributeImage, displayString, null, null));
						}
					}
					break;
				}
			}
		}
		
		return (ICompletionProposal[])proposals.toArray(new ICompletionProposal[proposals.size()]);
	}

	public IContextInformation[] computeContextInformation(ITextViewer viewer, int offset) {
		return null;
	}

	public char[] getCompletionProposalAutoActivationCharacters() {
		return null;
	}

	public char[] getContextInformationAutoActivationCharacters() {
		return null;
	}

	public String getErrorMessage() {
		return null;
	}

	public IContextInformationValidator getContextInformationValidator() {
		return null;
	}
	
	private AssistInfo getAssistInfo(String source, int offset){
		StringBuffer matchString = new StringBuffer();
		String tagName = "";
		for(int i=0;i<offset;i++){
			char c = source.charAt(i);
			if(!Character.isWhitespace(c) && c!='=' && c!='"'){
				matchString.append(c);
			} else {
				if(matchString.length() > 0 && matchString.charAt(0)=='<'){
					tagName = matchString.toString().substring(1);
				}
				matchString.setLength(0);
			}
		}
		AssistInfo info = new AssistInfo();
		info.matchString = matchString.toString();
		info.tagName = tagName;
		return info;
	}
	
	/** XMLDTD擾邽߂̐K\ */
	private final Pattern patternDoctypePublic
		= Pattern.compile("<!DOCTYPE[\\s\r\n]+?[^<]+?[\\s\r\n]+?PUBLIC[\\s\r\n]*?\"(.+?)\"[\\s\r\n]*?\"(.+?)\".*?>",Pattern.DOTALL);
	private final Pattern patternDoctypeSystem
		= Pattern.compile("<!DOCTYPE[\\s\r\n]+?[^<]+?[\\s\r\n]+?SYSTEM[\\s\r\n]*?\"(.+?)\".*?>",Pattern.DOTALL);
	
	/** XMLŎgpĂDTD擾 */
	private String getDTD(String source){
		// PUBLIC Identifier
		Matcher matcher = patternDoctypePublic.matcher(source);
		if(matcher.find()){
			return matcher.group(2);
		}
		// SYSTEM Identifier
		matcher = patternDoctypeSystem.matcher(source);
		if(matcher.find()){
			return matcher.group(1);
		}
		return "";
	}	
	
	/** DTD⊮擾 */
	private void refresh(Reader in){
		
		tagList.clear();
		
		try {
			DTDParser parser = new DTDParser(in);
			DTD dtd = parser.parse();
			Object[] obj = dtd.getItems();
			for(int i=0;i<obj.length;i++){
				if(obj[i] instanceof DTDElement){
					DTDElement element = (DTDElement)obj[i];
					String name = element.getName();
					DTDItem item = element.getContent();
					boolean hasBody = true;
					if(item instanceof DTDEmpty){
						hasBody = false;
					}
					TagInfo tagInfo = new TagInfo();
					tagInfo.name = name;
					tagInfo.hasBody = hasBody;
					Iterator ite = element.attributes.keySet().iterator();
					
					while(ite.hasNext()){
						String attrName = (String)ite.next();
						DTDAttribute attr = element.getAttribute(attrName);
						
						DTDDecl decl = attr.getDecl();
						boolean required = false;
						if(decl == DTDDecl.REQUIRED){
							required = true;
						}
						
						AttributeInfo attrInfo = new AttributeInfo();
						attrInfo.name = attrName;
						attrInfo.required = required;
						tagInfo.attributes.add(attrInfo);
						
					}
					tagList.add(tagInfo);
				}
			}
		} catch(DTDParseException ex){
			// ignore
		} catch(Exception ex){
			ex.printStackTrace();
		}
	}
	
	/** ݂̃Lbg̈ʒũeLXg */
	private class AssistInfo {
		private String matchString;
		private String tagName;
	}
	
	/** ^Ȍ */
	private class TagInfo {
		private String name;
		private boolean hasBody;
		private List attributes = new ArrayList();
	}
	
	/** ̏ */
	private class AttributeInfo {
		private String name;
		private boolean required;
	}

}
