package jp.sf.amateras.xmleditor.editors;

import jp.sf.amateras.xmleditor.XMLEditorPlugin;

import org.eclipse.core.resources.IProject;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.jdt.core.IJavaProject;
import org.eclipse.jdt.core.JavaCore;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.preference.IPreferenceStore;
import org.eclipse.jface.text.TextViewer;
import org.eclipse.jface.text.hyperlink.IHyperlinkDetector;
import org.eclipse.jface.text.hyperlink.URLHyperlinkDetector;
import org.eclipse.jface.util.IPropertyChangeListener;
import org.eclipse.jface.util.PropertyChangeEvent;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.ui.IFileEditorInput;
import org.eclipse.ui.editors.text.TextEditor;
import org.eclipse.ui.texteditor.ContentAssistAction;
import org.eclipse.ui.texteditor.ITextEditorActionDefinitionIds;
import org.eclipse.ui.views.contentoutline.IContentOutlinePage;

public class XMLEditor extends TextEditor implements IPropertyChangeListener {

	private XMLOutlinePage outlinePage;
	private ColorManager colorManager;
	
	public XMLEditor() {
		super();
		colorManager = new ColorManager();
		setSourceViewerConfiguration(new XMLConfiguration(colorManager));
		setDocumentProvider(new XMLDocumentProvider());
		
		// ݒ̕ύXʒm󂯎邽߂̃Xio^
		IPreferenceStore store = XMLEditorPlugin.getDefault().getPreferenceStore();
		store.addPropertyChangeListener(this);
	}
	
	public void dispose() {
		if(outlinePage!=null){
			outlinePage.dispose();
		}
		colorManager.dispose();
		
		super.dispose();
		
		// ݒ̕ύXʒm󂯎邽߂̃Xi폜
		IPreferenceStore store = XMLEditorPlugin.getDefault().getPreferenceStore();
		store.removePropertyChangeListener(this);
	}

	public void propertyChange(PropertyChangeEvent event) {
		// ݒ̕ύXʒm
		XMLConfiguration config = (XMLConfiguration)getSourceViewerConfiguration();
		config.updatePreferences();
		// eLXgGfB^ĕ`
		getSourceViewer().invalidateTextPresentation();
	}
	
	public void doSave(IProgressMonitor progressMonitor) {
		super.doSave(progressMonitor);
		outlinePage.refresh();
	}

	public void doSaveAs() {
		super.doSaveAs();
		outlinePage.refresh();
	}

	public void createPartControl(Composite parent) {
		super.createPartControl(parent);
		
		IFileEditorInput input = (IFileEditorInput)getEditorInput();
		IProject project = input.getFile().getProject();
		IJavaProject javaProject = JavaCore.create(project);
		
		ClassNameHyperlinkDetector detector = new ClassNameHyperlinkDetector(javaProject);
		TextViewer textViewer = (TextViewer)getSourceViewer();
		textViewer.setHyperlinkDetectors(
				new IHyperlinkDetector[]{ new URLHyperlinkDetector(textViewer), detector }, SWT.CTRL);
	}

	public Object getAdapter(Class adapter) {
		if(IContentOutlinePage.class.equals(adapter)){
			if(outlinePage==null){
				outlinePage = new XMLOutlinePage(this);
			}
			return outlinePage;
		}
		return super.getAdapter(adapter);
	}
	
	protected void createActions() {
		super.createActions();
		IAction action = new ContentAssistAction(
				XMLEditorPlugin.getResourceBundle(),"ContentAssistProposal", this);
		action.setActionDefinitionId(ITextEditorActionDefinitionIds.CONTENT_ASSIST_PROPOSALS);
		setAction("ContentAssistProposal", action);
	}
	
}
